@@ -306,17 +306,71 @@ is_verbose(void)
 /* Fake tgoto call - very crapy, but works for our needs */
 #ifndef HAVE_TERMCAP
 static char *
-tgoto(const char *cap, int a, int b)
+tgoto(const char *cap, int col, int line)
 {
 	static char buf[20];
+	char *p, *e, c, dbuf[6];
+	int oncol = 0, which = line, i;
 
-	snprintf(buf, sizeof(buf), cap, b, a);
+	p = buf;
+	e = p + sizeof(buf);
+	while ((c = *cap++)) {
+		if (c != '%' || ((c = *cap++) == '%')) {
+			*p++ = c;
+			if (p >= e) {
+				errno = E2BIG;
+				return NULL;
+			}
+			continue;
+		}
+		switch (c) {
+		case '3':
+		case '2':
+		case 'd':
+			i = 0;
+			do
+				dbuf[i++] = which % 10 | '0';
+			while ((which /= 10));
+			if (c != 'd') {
+				c -= '0';
+				if (i > c) {
+					errno = EINVAL;
+					return NULL;
+				}
+				while (i < c)
+					dbuf[i++] = '0';
+			}
+			if (p + i >= e) {
+				errno = E2BIG;
+				return NULL;
+			}
+			do
+				*p++ = dbuf[--i];
+			while (i);
+			break;
+		case 'r':
+			oncol = 0;
+			break;
+		case 'i':
+			col++;
+			line++;
+			which++;
+			continue;
+		default:
+			errno = EINVAL;
+			return NULL;
+		}
+
+		oncol = 1 - oncol;
+		which = oncol ? col : line;
+	}
+	*p = '\0';
 	return buf;
 }
 #endif
 
 static bool
-colour_terminal(FILE * __EINFO_RESTRICT f)
+colour_terminal(FILE * EINFO_RESTRICT f)
 {
 	static int in_colour = -1;
 	char *e, *ee, *end, *d, *p;
@@ -460,7 +514,7 @@ colour_terminal(FILE * __EINFO_RESTRICT f)
 }
 
 static int
-get_term_columns(FILE * __EINFO_RESTRICT stream)
+get_term_columns(FILE * EINFO_RESTRICT stream)
 {
 	struct winsize ws;
 	char *env = getenv("COLUMNS");
@@ -480,14 +534,14 @@ get_term_columns(FILE * __EINFO_RESTRICT stream)
 }
 
 void
-eprefix(const char *__EINFO_RESTRICT prefix)
+eprefix(const char *EINFO_RESTRICT prefix)
 {
 	_eprefix = prefix;
 }
 hidden_def(eprefix)
 
-static void
-elogv(int level, const char *__EINFO_RESTRICT fmt, va_list ap)
+static void EINFO_PRINTF(2, 0)
+elogv(int level, const char *EINFO_RESTRICT fmt, va_list ap)
 {
 	char *e = getenv("EINFO_LOG");
 	va_list apc;
@@ -503,7 +557,7 @@ elogv(int level, const char *__EINFO_RESTRICT fmt, va_list ap)
 }
 
 void
-elog(int level, const char *__EINFO_RESTRICT fmt, ...)
+elog(int level, const char *EINFO_RESTRICT fmt, ...)
 {
 	va_list ap;
 
@@ -514,7 +568,7 @@ elog(int level, const char *__EINFO_RESTRICT fmt, ...)
 hidden_def(elog)
 
 static int
-_eindent(FILE * __EINFO_RESTRICT stream)
+_eindent(FILE * EINFO_RESTRICT stream)
 {
 	char *env = getenv("EINFO_INDENT");
 	int amount = 0;
@@ -538,7 +592,7 @@ _eindent(FILE * __EINFO_RESTRICT stream)
 }
 
 static const char *
-_ecolor(FILE * __EINFO_RESTRICT f, ECOLOR color)
+_ecolor(FILE * EINFO_RESTRICT f, ECOLOR color)
 {
 	unsigned int i;
 
@@ -575,9 +629,8 @@ ecolor(ECOLOR color)
 	setenv("EINFO_LASTCMD", _cmd, 1); \
 }
 
-static int
-_einfo(FILE *f, ECOLOR color, const char *__EINFO_RESTRICT fmt, va_list va)
-
+static int EINFO_PRINTF(3, 0)
+_einfo(FILE *f, ECOLOR color, const char *EINFO_RESTRICT fmt, va_list va)
 {
 	int retval = 0;
 	char *last = getenv("EINFO_LASTCMD");
@@ -590,7 +643,7 @@ _einfo(FILE *f, ECOLOR color, const char *__EINFO_RESTRICT fmt, va_list va)
 		fprintf(f, "\n");
 	if (_eprefix)
 		fprintf(f, "%s%s%s|", _ecolor(f, color), _eprefix, _ecolor(f, ECOLOR_NORMAL));
-	fprintf(f, " %s*%s ", _ecolor(f, color), _ecolor(f, ECOLOR_NORMAL));
+	fprintf(f, "%s*%s ", _ecolor(f, color), _ecolor(f, ECOLOR_NORMAL));
 	retval += _eindent(f);
 	va_copy(ap, va);
 	retval += vfprintf(f, fmt, ap) + 3;
@@ -605,7 +658,7 @@ _einfo(FILE *f, ECOLOR color, const char *__EINFO_RESTRICT fmt, va_list va)
 #define _eerrorvn(fmt, ap) _einfo(stderr, ECOLOR_BAD, fmt, ap)
 
 int
-einfon(const char *__EINFO_RESTRICT fmt, ...)
+einfon(const char *EINFO_RESTRICT fmt, ...)
 {
 	int retval;
 	va_list ap;
@@ -621,7 +674,7 @@ einfon(const char *__EINFO_RESTRICT fmt, ...)
 hidden_def(einfon)
 
 int
-ewarnn(const char *__EINFO_RESTRICT fmt, ...)
+ewarnn(const char *EINFO_RESTRICT fmt, ...)
 {
 	int retval;
 	va_list ap;
@@ -637,7 +690,7 @@ ewarnn(const char *__EINFO_RESTRICT fmt, ...)
 hidden_def(ewarnn)
 
 int
-eerrorn(const char *__EINFO_RESTRICT fmt, ...)
+eerrorn(const char *EINFO_RESTRICT fmt, ...)
 {
 	int retval;
 	va_list ap;
@@ -651,7 +704,7 @@ eerrorn(const char *__EINFO_RESTRICT fmt, ...)
 hidden_def(eerrorn)
 
 int
-einfo(const char *__EINFO_RESTRICT fmt, ...)
+einfo(const char *EINFO_RESTRICT fmt, ...)
 {
 	int retval;
 	va_list ap;
@@ -668,7 +721,7 @@ einfo(const char *__EINFO_RESTRICT fmt, ...)
 hidden_def(einfo)
 
 int
-ewarn(const char *__EINFO_RESTRICT fmt, ...)
+ewarn(const char *EINFO_RESTRICT fmt, ...)
 {
 	int retval;
 	va_list ap;
@@ -686,7 +739,7 @@ ewarn(const char *__EINFO_RESTRICT fmt, ...)
 hidden_def(ewarn)
 
 void
-ewarnx(const char *__EINFO_RESTRICT fmt, ...)
+ewarnx(const char *EINFO_RESTRICT fmt, ...)
 {
 	int retval;
 	va_list ap;
@@ -703,7 +756,7 @@ ewarnx(const char *__EINFO_RESTRICT fmt, ...)
 hidden_def(ewarnx)
 
 int
-eerror(const char *__EINFO_RESTRICT fmt, ...)
+eerror(const char *EINFO_RESTRICT fmt, ...)
 {
 	int retval;
 	va_list ap;
@@ -721,7 +774,7 @@ eerror(const char *__EINFO_RESTRICT fmt, ...)
 hidden_def(eerror)
 
 void
-eerrorx(const char *__EINFO_RESTRICT fmt, ...)
+eerrorx(const char *EINFO_RESTRICT fmt, ...)
 {
 	va_list ap;
 
@@ -737,7 +790,7 @@ eerrorx(const char *__EINFO_RESTRICT fmt, ...)
 hidden_def(eerrorx)
 
 int
-ebegin(const char *__EINFO_RESTRICT fmt, ...)
+ebegin(const char *EINFO_RESTRICT fmt, ...)
 {
 	int retval;
 	va_list ap;
@@ -747,7 +800,7 @@ ebegin(const char *__EINFO_RESTRICT fmt, ...)
 	va_start(ap, fmt);
 	retval = _einfovn(fmt, ap);
 	va_end(ap);
-	retval += printf(" ...");
+	retval += printf("...");
 	if (colour_terminal(stdout))
 		retval += printf("\n");
 	LASTCMD("ebegin");
@@ -756,7 +809,7 @@ ebegin(const char *__EINFO_RESTRICT fmt, ...)
 hidden_def(ebegin)
 
 static void
-_eend(FILE * __EINFO_RESTRICT fp, int col, ECOLOR color, const char *msg)
+_eend(FILE * EINFO_RESTRICT fp, int col, ECOLOR color, const char *msg)
 {
 	int i;
 	int cols;
@@ -793,8 +846,8 @@ _eend(FILE * __EINFO_RESTRICT fp, int col, ECOLOR color, const char *msg)
 	}
 }
 
-static int
-_do_eend(const char *cmd, int retval, const char *__EINFO_RESTRICT fmt, va_list ap)
+static int EINFO_PRINTF(3, 0)
+_do_eend(const char *cmd, int retval, const char *EINFO_RESTRICT fmt, va_list ap)
 {
 	int col = 0;
 	FILE *fp = stdout;
@@ -817,7 +870,7 @@ _do_eend(const char *cmd, int retval, const char *__EINFO_RESTRICT fmt, va_list
 }
 
 int
-eend(int retval, const char *__EINFO_RESTRICT fmt, ...)
+eend(int retval, const char *EINFO_RESTRICT fmt, ...)
 {
 	va_list ap;
 
@@ -832,7 +885,7 @@ eend(int retval, const char *__EINFO_RESTRICT fmt, ...)
 hidden_def(eend)
 
 int
-ewend(int retval, const char *__EINFO_RESTRICT fmt, ...)
+ewend(int retval, const char *EINFO_RESTRICT fmt, ...)
 {
 	va_list ap;
 
@@ -900,7 +953,7 @@ void eoutdent(void)
 hidden_def(eoutdent)
 
 int
-einfovn(const char *__EINFO_RESTRICT fmt, ...)
+einfovn(const char *EINFO_RESTRICT fmt, ...)
 {
 	int retval;
 	va_list ap;
@@ -916,7 +969,7 @@ einfovn(const char *__EINFO_RESTRICT fmt, ...)
 hidden_def(einfovn)
 
 int
-ewarnvn(const char *__EINFO_RESTRICT fmt, ...)
+ewarnvn(const char *EINFO_RESTRICT fmt, ...)
 {
 	int retval;
 	va_list ap;
@@ -932,7 +985,7 @@ ewarnvn(const char *__EINFO_RESTRICT fmt, ...)
 hidden_def(ewarnvn)
 
 int
-einfov(const char *__EINFO_RESTRICT fmt, ...)
+einfov(const char *EINFO_RESTRICT fmt, ...)
 {
 	int retval;
 	va_list ap;
@@ -949,7 +1002,7 @@ einfov(const char *__EINFO_RESTRICT fmt, ...)
 hidden_def(einfov)
 
 int
-ewarnv(const char *__EINFO_RESTRICT fmt, ...)
+ewarnv(const char *EINFO_RESTRICT fmt, ...)
 {
 	int retval;
 	va_list ap;
@@ -966,7 +1019,7 @@ ewarnv(const char *__EINFO_RESTRICT fmt, ...)
 hidden_def(ewarnv)
 
 int
-ebeginv(const char *__EINFO_RESTRICT fmt, ...)
+ebeginv(const char *EINFO_RESTRICT fmt, ...)
 {
 	int retval;
 	va_list ap;
@@ -976,7 +1029,7 @@ ebeginv(const char *__EINFO_RESTRICT fmt, ...)
 
 	va_start(ap, fmt);
 	retval = _einfovn(fmt, ap);
-	retval += printf(" ...");
+	retval += printf("...");
 	if (colour_terminal(stdout))
 		retval += printf("\n");
 	va_end(ap);
@@ -986,7 +1039,7 @@ ebeginv(const char *__EINFO_RESTRICT fmt, ...)
 hidden_def(ebeginv)
 
 int
-eendv(int retval, const char *__EINFO_RESTRICT fmt, ...)
+eendv(int retval, const char *EINFO_RESTRICT fmt, ...)
 {
 	va_list ap;
 
@@ -1001,7 +1054,7 @@ eendv(int retval, const char *__EINFO_RESTRICT fmt, ...)
 hidden_def(eendv)
 
 int
-ewendv(int retval, const char *__EINFO_RESTRICT fmt, ...)
+ewendv(int retval, const char *EINFO_RESTRICT fmt, ...)
 {
 	va_list ap;
 
