@@ -9,7 +9,7 @@
  */
 
 /*
- * Copyright 2007-2008 Roy Marples <roy@marples.name>
+ * Copyright 2007-2009 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -842,10 +842,14 @@ main(int argc, char **argv)
 	}
 
 	newlevel = argv[optind++];
-	/* For compat with old system */
+	/* To make life easier, we only have the shutdown runlevel as
+	 * nothing really needs to know that we're rebooting.
+	 * But for those that do, you can test against RC_REBOOT. */
 	if (newlevel) {
-		if (strcmp(newlevel, "reboot") == 0)
+		if (strcmp(newlevel, "reboot") == 0) {
 			newlevel = UNCONST(RC_LEVEL_SHUTDOWN);
+			setenv("RC_REBOOT", "YES", 1);
+		}
 	}
 
 	/* Enable logging */
@@ -944,8 +948,10 @@ main(int argc, char **argv)
 	}
 
 	/* Load our deptree */
-	if ((deptree = _rc_deptree_load(&regen)) == NULL)
+	if ((deptree = _rc_deptree_load(0, &regen)) == NULL)
 		eerrorx("failed to load deptree");
+	if (exists(RC_DEPTREE_SKEWED))
+		ewarn("WARNING: clock skew detected!");
 
 	/* Clean the failed services state dir */
 	clean_failed();
@@ -1062,7 +1068,7 @@ main(int argc, char **argv)
 
 #ifdef __linux__
 	/* mark any services skipped as started */
-	proc = p = proc_getent("noinitd");
+	proc = p = proc_getent("noinit");
 	if (proc) {
 		while ((token = strsep(&p, ",")))
 			rc_service_mark(token, RC_SERVICE_STARTED);
@@ -1083,7 +1089,7 @@ main(int argc, char **argv)
 
 #ifdef __linux__
 	/* mark any services skipped as stopped */
-	proc = p = proc_getent("noinitd");
+	proc = p = proc_getent("noinit");
 	if (proc) {
 		while ((token = strsep(&p, ",")))
 			rc_service_mark(token, RC_SERVICE_STOPPED);
