@@ -8,7 +8,7 @@
    */
 
 /*
- * Copyright 2007-2008 Roy Marples <roy@marples.name>
+ * Copyright 2007-2009 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -645,8 +645,6 @@ int start_stop_daemon(int argc, char **argv)
 	RC_STRINGLIST *env_list;
 	RC_STRING *env;
 	char *tmp, *newpath, *np;
-	bool sethome = false;
-	bool setuser = false;
 	char *p;
 	char *token;
 	char exec_file[PATH_MAX];
@@ -655,8 +653,7 @@ int start_stop_daemon(int argc, char **argv)
 	char line[130];
 	FILE *fp;
 	size_t len;
-	bool setumask = false;
-	mode_t numask;
+	mode_t numask = 022;
 	char **margv;
 	unsigned int start_wait = 0;
 
@@ -674,12 +671,19 @@ int start_stop_daemon(int argc, char **argv)
 			eerror("%s: invalid nice level `%s' (SSD_NICELEVEL)",
 				applet, tmp);
 
-	/* Get our initial dir */
+	/* Get our user name and initial dir */
+	p = getenv("USER");
 	home = getenv("HOME");
-	if (!home) {
+	if (home == NULL || p == NULL) {
 		pw = getpwuid(getuid());
-		if (pw)
-			home = pw->pw_dir;
+		if (pw != NULL) {
+			if (p == NULL)
+				setenv("USER", pw->pw_name, 1);
+			if (home == NULL) {
+				setenv("HOME", pw->pw_dir, 1);
+				home = pw->pw_dir;
+			}
+		}
 	}
 
 	while ((opt = getopt_long(argc, argv, getoptstring, longopts,
@@ -715,14 +719,20 @@ int start_stop_daemon(int argc, char **argv)
 				if (sscanf(tmp, "%d", &tid) != 1)
 					pw = getpwnam(tmp);
 				else
-					pw = getpwuid((uid_t) tid);
+					pw = getpwuid((uid_t)tid);
 
-				if (!pw)
+				if (pw == NULL)
 					eerrorx("%s: user `%s' not found",
 						applet, tmp);
 				uid = pw->pw_uid;
 				home = pw->pw_dir;
-				if (!gid)
+				unsetenv("HOME");
+				if (pw->pw_dir)
+					setenv("HOME", pw->pw_dir, 1);
+				unsetenv("USER");
+				if (pw->pw_name)
+					setenv("USER", pw->pw_name, 1);
+				if (gid == 0)
 					gid = pw->pw_gid;
 
 				if (p) {
@@ -732,7 +742,7 @@ int start_stop_daemon(int argc, char **argv)
 					else
 						gr = getgrgid((gid_t) tid);
 
-					if (!gr)
+					if (gr == NULL)
 						eerrorx("%s: group `%s'"
 							" not found",
 							applet, tmp);
@@ -746,34 +756,24 @@ int start_stop_daemon(int argc, char **argv)
 			break;
 
 		case 'e': /* --env */
-			if (putenv(optarg) == 0) {
-				if (strncmp("HOME=", optarg, 5) == 0) {
-					sethome = true;
-					home = strchr(optarg, '=') + 1;
-				} else if (strncmp("USER=", optarg, 5) == 0)
-					setuser = true;
-			}
+			putenv(optarg);
 			break;
 
 		case 'g':  /* --group <group>|<gid> */
-			{
-				if (sscanf(optarg, "%d", &tid) != 1)
-					gr = getgrnam(optarg);
-				else
-					gr = getgrgid((gid_t) tid);
-
-				if (!gr)
-					eerrorx("%s: group `%s' not found",
-						applet, optarg);
-				gid = gr->gr_gid;
-			}
+			if (sscanf(optarg, "%d", &tid) != 1)
+				gr = getgrnam(optarg);
+			else
+				gr = getgrgid((gid_t)tid);
+			if (gr == NULL)
+				eerrorx("%s: group `%s' not found",
+					applet, optarg);
+			gid = gr->gr_gid;
 			break;
 
 		case 'k':
 			if (parse_mode(&numask, optarg))
 				eerrorx("%s: invalid mode `%s'",
 					applet, optarg);
-			setumask = true;
 			break;
 
 		case 'm':  /* --make-pidfile */
@@ -1039,8 +1039,7 @@ int start_stop_daemon(int argc, char **argv)
 	/* Child process - lets go! */
 	if (pid == 0) {
 		pid_t mypid = getpid();
-		if (setumask)
-			umask(numask);
+		umask(numask);
 
 #ifdef TIOCNOTTY
 		tty_fd = open("/dev/tty", O_RDWR);
@@ -1089,23 +1088,8 @@ int start_stop_daemon(int argc, char **argv)
 			eerrorx("%s: unable to set groupid to %d", applet, gid);
 		if (changeuser && initgroups(changeuser, gid))
 			eerrorx("%s: initgroups (%s, %d)", applet, changeuser, gid);
-		if (uid) {
-			if (setuid(uid))
-				eerrorx ("%s: unable to set userid to %d", applet, uid);
-			pw = getpwuid(uid);
-			if (pw) {
-				if (!sethome) {
-					unsetenv("HOME");
-					if (pw->pw_dir)
-						setenv("HOME", pw->pw_dir, 1);
-				}
-				if (!setuser) {
-					unsetenv("USER");
-					if (pw->pw_name)
-						setenv("USER", pw->pw_name, 1);
-				}
-			}
-		}
+		if (uid && setuid(uid))
+			eerrorx ("%s: unable to set userid to %d", applet, uid);
 
 		/* Close any fd's to the passwd database */
 		endpwent();
@@ -1174,8 +1158,6 @@ int start_stop_daemon(int argc, char **argv)
 			setenv("PATH", newpath, 1);
 		}
 
-		umask(022);
-
 		stdout_fd = devnull_fd;
 		stderr_fd = devnull_fd;
 		if (redirect_stdout) {
